unit FormMain;

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, Menus,
  ExtCtrls, Engine, LCLType, Types, Translator, ApplicationInfo,
  RegistryEx, PersistentForm, FormImages;

type

  { TFormMain }

  TFormMain = class(TForm)
    ApplicationInfo1: TApplicationInfo;
    PaintBox1: TPaintBox;
    Timer1: TTimer;
    procedure FormClose(Sender: TObject; var CloseAction: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: char);
    procedure FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormShow(Sender: TObject);
    procedure PaintBox1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaintBox1MouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure PaintBox1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure PaintBox1MouseWheelDown(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure PaintBox1MouseWheelUp(Sender: TObject; Shift: TShiftState;
      MousePos: TPoint; var Handled: Boolean);
    procedure PaintBox1Paint(Sender: TObject);
    procedure PaintBox1Resize(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
  private
    Initialized: Boolean;
    procedure FullScreenChangeExecute(Sender: TObject; Active: Boolean);
    procedure ExitExecute(Sender: TObject);
    procedure LoadConfig;
    procedure SaveConfig;
    procedure LoadGraphicAssets;
    procedure DarkModeChangeExecute(Sender: TObject);
    procedure CopyImage(DestBmp: TBitmap; SourceBmp: TBitmap; Grayed: Boolean = False);
  public
    Engine: TEngine;
    procedure EraseBackground(DC: HDC); override;
  end;


implementation

{$R *.lfm}

uses
  PixelPointer, Core, FormTest, TestCases;

{ TFormMain }

procedure TFormMain.FormCreate(Sender: TObject);
{$IFDEF UNIX}
const
  UnixLanguagesDir = '/usr/share/TransLines/languages';
  {$ENDIF}
begin
  {$IFDEF UNIX}
  // If installed in UNIX system then use installation directory for po files
  if not DirectoryExists(Core.Translator1.POFilesFolder) and DirectoryExists(UnixLanguagesDir) then
    Core.Translator1.POFilesFolder := UnixLanguagesDir;
  {$ENDIF}

  Engine := TEngine.Create(nil);
  Engine.RegistryContext := ApplicationInfo1.GetRegistryContext;
  Engine.Translator := Core.Core.Translator1;
  Engine.OnFullScreenChange := FullScreenChangeExecute;
  Engine.OnExit := ExitExecute;
  LoadConfig;
  // Load cities after language core is loaded to have city names translated
  Engine.InitCities;
  Engine.LoadFromRegistry;
  //PaintBox1.ControlStyle := [csOpaque];
  Randomize;
end;

procedure TFormMain.FormDestroy(Sender: TObject);
begin
  SaveConfig;
  FreeAndNil(Engine);
end;

procedure TFormMain.FormShow(Sender: TObject);
begin
  if not Initialized then begin
    Initialized := True;
    Core.Core.PersistentForm1.RegistryContext := ApplicationInfo1.GetRegistryContext;
    Core.Core.PersistentForm1.Load(Self, False, True);
    Engine.FullScreen := Core.Core.PersistentForm1.FormFullScreen;
    Engine.InitMenus;
    Engine.OnDarkModeChange := DarkModeChangeExecute;
    Engine.Map.Size := Point(PaintBox1.Width, PaintBox1.Height);
    Engine.View.DestRect := Rect(0, 0, PaintBox1.Width, PaintBox1.Height);
    Engine.MainMenu;
    LoadGraphicAssets;
  end;
end;

procedure TFormMain.FormClose(Sender: TObject; var CloseAction: TCloseAction);
begin
  Core.Core.PersistentForm1.Save(Self);
  Application.Terminate;
end;

procedure TFormMain.FormKeyPress(Sender: TObject; var Key: char);
begin
  if Key = #32 then begin
    if Engine.State = gsRunning then Engine.State := gsPaused
    else if Engine.State = gsPaused then Engine.State := gsRunning;
  end;
end;

procedure TFormMain.FormKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState
  );
const
  KeyF11 = 122;
{$IFDEF DEBUG}
  KeyF12 = 123;
{$ENDIF}
var
  FormTest: TFormTest;
begin
  if Key = KeyF11 then begin
    Engine.FullScreen := not Engine.FullScreen;
  end else
{$IFDEF DEBUG}
  if Key = KeyF12 then begin
    FormTest := TFormTest.Create(nil);
    try
      FormTest.TestCases := InitTestCases;
      try
        FormTest.ShowModal;
      finally
        FormTest.TestCases.Free;
      end;
    finally
      FormTest.Free;
    end;
  end else
{$ENDIF}
    Engine.KeyUp(Key);
end;

procedure TFormMain.PaintBox1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  Engine.MouseDown(Button, Point(X, Y));
end;

procedure TFormMain.PaintBox1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  Engine.MouseMove(Point(X, Y));
end;

procedure TFormMain.PaintBox1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  Engine.MouseUp(Button, Point(X, Y));
end;

procedure TFormMain.PaintBox1MouseWheelDown(Sender: TObject;
  Shift: TShiftState; MousePos: TPoint; var Handled: Boolean);
begin
  Engine.View.Zoom := Engine.View.Zoom / 1.2;
end;

procedure TFormMain.PaintBox1MouseWheelUp(Sender: TObject; Shift: TShiftState;
  MousePos: TPoint; var Handled: Boolean);
begin
  Engine.View.Zoom := Engine.View.Zoom * 1.2;
end;

procedure TFormMain.PaintBox1Paint(Sender: TObject);
begin
  Engine.View.DestRect := Rect(0, 0, PaintBox1.Width, PaintBox1.Height);
  Engine.Paint(PaintBox1.Canvas, Point(PaintBox1.Width, PaintBox1.Height));
end;

procedure TFormMain.PaintBox1Resize(Sender: TObject);
begin
  Engine.Redraw;
end;

procedure TFormMain.Timer1Timer(Sender: TObject);
begin
  Engine.Tick;
  if Engine.RedrawPending then
    PaintBox1.Repaint;
end;

procedure TFormMain.FullScreenChangeExecute(Sender: TObject; Active: Boolean);
begin
  Core.Core.PersistentForm1.SetFullScreen(Active);
end;

procedure TFormMain.ExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TFormMain.EraseBackground(DC: HDC);
begin
  // Do nothing, all background space covered by controls
end;

procedure TFormMain.LoadConfig;
var
  LangCode: string;
begin
  with TRegistryEx.Create do
  try
    CurrentContext := ApplicationInfo1.GetRegistryContext;

    if ValueExists('LanguageCode') then begin
      LangCode := ReadStringWithDefault('LanguageCode', '');
      Core.Core.Translator1.Language := Core.Core.Translator1.Languages.SearchByCode(LangCode);
    end else Core.Core.Translator1.Language := Core.Core.Translator1.Languages.SearchByCode('');
    Engine.LoadFromRegistry;
  finally
    Free;
  end;
end;

procedure TFormMain.SaveConfig;
begin
  with TRegistryEx.Create do
  try
    CurrentContext := ApplicationInfo1.GetRegistryContext;

    if Assigned(Core.Core.Translator1.Language) and (Core.Core.Translator1.Language.Code <> '') then
      WriteString('LanguageCode', Core.Core.Translator1.Language.Code)
      else DeleteValue('LanguageCode');
    Engine.SaveToRegistry;
  finally
    Free;
  end;
end;

procedure TFormMain.LoadGraphicAssets;
var
  FormImages: TFormImages;
begin
  FormImages := TFormImages.Create(nil);

  CopyImage(Engine.ImagePassenger.Bitmap, FormImages.ImagePassenger.Picture.Bitmap);
  CopyImage(Engine.ImagePassenger.BitmapDisabled, FormImages.ImagePassenger.Picture.Bitmap, True);
  CopyImage(Engine.ImageLocomotive.Bitmap, FormImages.ImageLocomotive.Picture.Bitmap);
  CopyImage(Engine.ImageLocomotive.BitmapDisabled, FormImages.ImageLocomotive.Picture.Bitmap, True);
  CopyImage(Engine.ImageLine.Bitmap, FormImages.ImageLine.Picture.Bitmap);
  CopyImage(Engine.ImageLine.BitmapDisabled, FormImages.ImageLine.Picture.Bitmap, True);
  CopyImage(Engine.ImageTerminal.Bitmap, FormImages.ImageTerminal.Picture.Bitmap);
  CopyImage(Engine.ImageTerminal.BitmapDisabled, FormImages.ImageTerminal.Picture.Bitmap, True);
  CopyImage(Engine.ImageTunnel.Bitmap, FormImages.ImageTunnel.Picture.Bitmap);
  CopyImage(Engine.ImageTunnel.BitmapDisabled, FormImages.ImageTunnel.Picture.Bitmap, True);
  CopyImage(Engine.ImageCarriage.Bitmap, FormImages.ImageCarriage.Picture.Bitmap);
  CopyImage(Engine.ImageCarriage.BitmapDisabled, FormImages.ImageCarriage.Picture.Bitmap, True);
  CopyImage(Engine.ButtonBack.Bitmap, FormImages.ImageLeftArrow.Picture.Bitmap);
  CopyImage(Engine.ButtonBack.BitmapDisabled, FormImages.ImageLeftArrow.Picture.Bitmap, True);
  CopyImage(Engine.ImagePlay.Bitmap, FormImages.ImagePlay.Picture.Bitmap);
  CopyImage(Engine.ImagePlay.BitmapDisabled, FormImages.ImagePlay.Picture.Bitmap, True);
  CopyImage(Engine.ImagePause.Bitmap, FormImages.ImagePause.Picture.Bitmap);
  CopyImage(Engine.ImagePause.BitmapDisabled, FormImages.ImagePause.Picture.Bitmap, True);
  CopyImage(Engine.ImageFastForward.Bitmap, FormImages.ImageFastForward.Picture.Bitmap);
  CopyImage(Engine.ImageFastForward.BitmapDisabled, FormImages.ImageFastForward.Picture.Bitmap, True);
  CopyImage(Engine.ImageAchievement.Bitmap, FormImages.ImageAchievement.Picture.Bitmap);
  CopyImage(Engine.ImageAchievement.BitmapDisabled, FormImages.ImageAchievement.Picture.Bitmap, True);
  if Engine.DarkMode then begin
    BitmapInvert(Engine.ImagePassenger.Bitmap);
    BitmapInvert(Engine.ImagePassenger.BitmapDisabled);
    BitmapInvert(Engine.ImageLocomotive.Bitmap);
    BitmapInvert(Engine.ImageLocomotive.BitmapDisabled);
    BitmapInvert(Engine.ImageLine.Bitmap);
    BitmapInvert(Engine.ImageLine.BitmapDisabled);
    BitmapInvert(Engine.ImageTerminal.Bitmap);
    BitmapInvert(Engine.ImageTerminal.BitmapDisabled);
    BitmapInvert(Engine.ImageTunnel.Bitmap);
    BitmapInvert(Engine.ImageTunnel.BitmapDisabled);
    BitmapInvert(Engine.ImageCarriage.Bitmap);
    BitmapInvert(Engine.ImageCarriage.BitmapDisabled);
    BitmapInvert(Engine.ButtonBack.Bitmap);
    BitmapInvert(Engine.ButtonBack.BitmapDisabled);
    BitmapInvert(Engine.ImagePlay.Bitmap);
    BitmapInvert(Engine.ImagePlay.BitmapDisabled);
    BitmapInvert(Engine.ImagePause.Bitmap);
    BitmapInvert(Engine.ImagePause.BitmapDisabled);
    BitmapInvert(Engine.ImageFastForward.Bitmap);
    BitmapInvert(Engine.ImageFastForward.BitmapDisabled);
    BitmapInvert(Engine.ImageAchievement.Bitmap);
    BitmapInvert(Engine.ImageAchievement.BitmapDisabled);
  end;
  Engine.ImageNewTrain.Bitmap.Assign(Engine.ImageLocomotive.Bitmap);
  Engine.ImageNewTrain.BitmapDisabled.Assign(Engine.ImageLocomotive.BitmapDisabled);

  FreeAndNil(FormImages);
end;

procedure TFormMain.DarkModeChangeExecute(Sender: TObject);
begin
  LoadGraphicAssets;
end;

procedure TFormMain.CopyImage(DestBmp: TBitmap; SourceBmp: TBitmap;
  Grayed: Boolean);
begin
  DestBmp.PixelFormat := SourceBmp.PixelFormat;
  DestBmp.SetSize(0, 0);
  DestBmp.SetSize(SourceBmp.Width, SourceBmp.Height);
  BitmapCopyRect(DestBmp, Bounds(0, 0, DestBmp.Width, DestBmp.Height), SourceBmp,
    Point(0, 0));
  if Grayed then BitmapBlendColor(DestBmp, $ffffffff);
end;

end.

